@@ -45,20 +45,6 @@ zone_is_nsec3_optout(ldns_rbtree_t *zone_nodes)
 	return 0;
 }
 
-static bool
-ldns_rr_list_contains_name(const ldns_rr_list *rr_list,
-					  const ldns_rdf *name)
-{
-	size_t i;
-	for (i = 0; i < ldns_rr_list_rr_count(rr_list); i++) {
-		if (ldns_dname_compare(name,
-		    ldns_rr_owner(ldns_rr_list_rr(rr_list, i))) == 0) {
-			return true;
-		}
-	}
-	return false;
-}
-
 static void
 print_type(ldns_rr_type type)
 {
@@ -87,13 +73,14 @@ create_dnssec_zone(ldns_zone *orig_zone)
 	   because the needed information is to be read later. in that case
 	   we keep a list of those nsec3's and retry to add them later */
 	ldns_rr_list *failed_nsec3s = ldns_rr_list_new();
+	ldns_rr_list *failed_nsec3_rrsigs = ldns_rr_list_new();
 
 	dnssec_zone = ldns_dnssec_zone_new();
-    	if (ldns_dnssec_zone_add_rr(dnssec_zone, ldns_zone_soa(orig_zone)) !=
-	    LDNS_STATUS_OK) {
+    	if (ldns_dnssec_zone_add_rr(dnssec_zone, ldns_zone_soa(orig_zone))
+		       	!= LDNS_STATUS_OK) {
 		if (verbosity > 0) {
-			fprintf(stderr,
-				   "Error adding SOA to dnssec zone, skipping record\n");
+			fprintf(stderr, "Error adding SOA to dnssec zone, "
+					"skipping record\n");
 		}
 	}
 
@@ -101,12 +88,27 @@ create_dnssec_zone(ldns_zone *orig_zone)
 		cur_rr = ldns_rr_list_rr(ldns_zone_rrs(orig_zone), i);
 		status = ldns_dnssec_zone_add_rr(dnssec_zone, cur_rr);
 		if (status != LDNS_STATUS_OK) {
-			if (status == LDNS_STATUS_DNSSEC_NSEC3_ORIGINAL_NOT_FOUND) {
-				ldns_rr_list_push_rr(failed_nsec3s, cur_rr);
+			if (LDNS_STATUS_DNSSEC_NSEC3_ORIGINAL_NOT_FOUND
+					== status) {
+				if (ldns_rr_get_type(cur_rr)
+					    == LDNS_RR_TYPE_RRSIG
+					    && ldns_rdf2rr_type(
+						  ldns_rr_rrsig_typecovered(
+							  cur_rr
+						   )
+						) == LDNS_RR_TYPE_NSEC3) {
+					ldns_rr_list_push_rr(
+							failed_nsec3_rrsigs, 
+							cur_rr);
+				} else {
+					ldns_rr_list_push_rr(failed_nsec3s, 
+							cur_rr);
+				}
 			} else {
 				if (verbosity > 0) {
-					fprintf(stderr, "Error adding RR to dnssec zone");
-					fprintf(stderr, ", skipping record:\n");
+					fprintf(stderr, "Error adding RR to "
+							"dnssec zone"
+							", skipping record:\n");
 					ldns_rr_print(stderr, cur_rr);
 				}
 			}
@@ -119,17 +121,24 @@ create_dnssec_zone(ldns_zone *orig_zone)
 			cur_rr = ldns_rr_list_rr(failed_nsec3s, i);
 			status = ldns_dnssec_zone_add_rr(dnssec_zone, cur_rr);
 		}
+		for (i = 0; i < ldns_rr_list_rr_count(failed_nsec3_rrsigs); 
+				i++) {
+			cur_rr = ldns_rr_list_rr(failed_nsec3_rrsigs, i);
+			status = ldns_dnssec_zone_add_rr(dnssec_zone, cur_rr);
+		}
 	}
 
+	ldns_rr_list_free(failed_nsec3_rrsigs);
 	ldns_rr_list_free(failed_nsec3s);
 	return dnssec_zone;
 }
 
 static ldns_status
-verify_dnssec_rrset(ldns_rdf *zone_name,
-					ldns_rdf *name,
-                    ldns_dnssec_rrsets *rrset,
-                    ldns_rr_list *keys)
+verify_dnssec_rrset(
+		ldns_rdf *zone_name, 
+		ldns_rdf *name,
+		ldns_dnssec_rrsets *rrset,
+		ldns_rr_list *keys) 
 {
 	ldns_rr_list *rrset_rrs;
 	ldns_dnssec_rrs *cur_rr, *cur_sig;
@@ -149,17 +158,16 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 	if (cur_sig) {
 		while (cur_sig) {
 			good_keys = ldns_rr_list_new();
-			status = ldns_verify_rrsig_keylist(rrset_rrs,
-										cur_sig->rr,
-										keys,
-										good_keys);
+			status = ldns_verify_rrsig_keylist(rrset_rrs, 
+					cur_sig->rr, keys, good_keys);
 			if (status != LDNS_STATUS_OK) {
 				if (verbosity > 0) {
 					printf("Error: %s",
-						  ldns_get_errorstr_by_id(status));
+						ldns_get_errorstr_by_id(
+							status));
 					printf(" for ");
 					ldns_rdf_print(stdout,
-								ldns_rr_owner(rrset->rrs->rr));
+						ldns_rr_owner(rrset->rrs->rr));
 					printf("\t");
 					print_type(rrset->type);
 					printf("\n");
@@ -172,9 +180,11 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 					}
 					if (verbosity >= 4) {
 						printf("RRSet:\n");
-						ldns_dnssec_rrs_print(stdout, rrset->rrs);
+						ldns_dnssec_rrs_print(stdout,
+								rrset->rrs);
 						printf("Signature:\n");
-						ldns_rr_print(stdout, cur_sig->rr);
+						ldns_rr_print(stdout,
+								cur_sig->rr);
 						printf("\n");
 					}
 				}
@@ -189,7 +199,8 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 			ldns_dname_compare(name, zone_name) == 0) {
 			if (verbosity > 0) {
 				printf("Error: no signatures for ");
-				ldns_rdf_print(stdout, ldns_rr_owner(rrset->rrs->rr));
+				ldns_rdf_print(stdout, 
+						ldns_rr_owner(rrset->rrs->rr));
 				printf("\t");
 				print_type(rrset->type);
 				printf("\n");
@@ -201,9 +212,10 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 }
 
 static ldns_status
-verify_single_rr(ldns_rr *rr,
-			  ldns_dnssec_rrs *signature_rrs,
-			  ldns_rr_list *keys)
+verify_single_rr(
+		ldns_rr *rr, 
+		ldns_dnssec_rrs *signature_rrs, 
+		ldns_rr_list *keys)
 {
 	ldns_rr_list *rrset_rrs;
 	ldns_rr_list *good_keys;
@@ -217,13 +229,12 @@ verify_single_rr(ldns_rr *rr,
 	cur_sig = signature_rrs;
 	while (cur_sig) {
 		good_keys = ldns_rr_list_new();
-		status = ldns_verify_rrsig_keylist(rrset_rrs,
-									cur_sig->rr,
-									keys,
-									good_keys);
+		status = ldns_verify_rrsig_keylist(
+				rrset_rrs, cur_sig->rr, keys, good_keys);
 		if (status != LDNS_STATUS_OK) {
 			if (verbosity >= 1) {
-				printf("Error: %s ", ldns_get_errorstr_by_id(status));
+				printf("Error: %s ", 
+					ldns_get_errorstr_by_id(status));
 				if (result == LDNS_STATUS_OK) {
 					result = status;
 				}
@@ -256,8 +267,7 @@ verify_single_rr(ldns_rr *rr,
 }
 
 static ldns_status
-verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
-                        ldns_dnssec_name *name)
+verify_next_hashed_name(ldns_rbtree_t *zone_nodes, ldns_dnssec_name *name)
 {
 	ldns_rbnode_t *next_node;
 	ldns_dnssec_name *next_name;
@@ -268,8 +278,8 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 	ldns_rdf *next_owner_dname;
 
 	if (!name->hashed_name) {
-		name->hashed_name = ldns_nsec3_hash_name_frm_nsec3(name->nsec,
-		                                                   name->name);
+		name->hashed_name = ldns_nsec3_hash_name_frm_nsec3(
+				name->nsec, name->name);
 	}
 	next_node = ldns_rbtree_first(zone_nodes);
 	while (next_node != LDNS_RBTREE_NULL) {
@@ -283,21 +293,21 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 		}
 		if (!next_name->hashed_name) {
 			next_name->hashed_name = ldns_nsec3_hash_name_frm_nsec3(
-			                              name->nsec, next_name->name);
+					name->nsec, next_name->name);
 		}
 		/* we keep track of what 'so far' is the next hashed name;
 		 * it must of course be 'larger' than the current name
 		 * if we find one that is larger, but smaller than what we
 		 * previously thought was the next one, that one is the next
 		 */
-		cmp = ldns_dname_compare(name->hashed_name,
-		                         next_name->hashed_name);
+		cmp = ldns_dname_compare(
+				name->hashed_name, next_name->hashed_name);
 		if (cmp < 0) {
 			if (!cur_next_name) {
 				cur_next_name = next_name;
 			} else {
 				cmp = ldns_dname_compare(next_name->hashed_name,
-				                         cur_next_name->hashed_name);
+						cur_next_name->hashed_name);
 				if (cmp < 0) {
 					cur_next_name = next_name;
 				}
@@ -309,8 +319,8 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 			cur_first_name = next_name;
 		} else {
 			cmp = ldns_dname_compare(next_name->hashed_name,
-									 cur_first_name->hashed_name);
-			if (cmp < 0) {
+					cur_first_name->hashed_name);
+		       	if (cmp < 0) {
 				cur_first_name = next_name;
 			}
 		}
@@ -342,9 +352,10 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 }
 
 static ldns_status
-verify_nsec(ldns_rbtree_t *zone_nodes,
-            ldns_rbnode_t *cur_node,
-            ldns_rr_list *keys)
+verify_nsec(
+		ldns_rbtree_t *zone_nodes, 
+		ldns_rbnode_t *cur_node, 
+		ldns_rr_list *keys)
 {
 	ldns_rbnode_t *next_node;
 	ldns_dnssec_name *name, *next_name;
@@ -355,8 +366,7 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 	if (name->nsec) {
 		if (name->nsec_signatures) {
 			status = verify_single_rr(name->nsec,
-								 name->nsec_signatures,
-								 keys);
+					name->nsec_signatures, keys);
 			if (result == LDNS_STATUS_OK) {
 				result = status;
 			}
@@ -376,21 +386,29 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 				/* simply try next name */
 				next_node = ldns_rbtree_next(cur_node);
 				if (next_node == LDNS_RBTREE_NULL) {
-					next_node = ldns_rbtree_first(zone_nodes);
+					next_node = ldns_rbtree_first(
+							zone_nodes);
 				}
-				next_node = ldns_dnssec_name_node_next_nonglue(next_node);
+				next_node = ldns_dnssec_name_node_next_nonglue(
+						next_node);
 				next_name = (ldns_dnssec_name *)next_node->data;
-				if (ldns_dname_compare(next_name->name,
-									   ldns_rr_rdf(name->nsec, 0))
-					!= 0) {
+				if (ldns_dname_compare(
+							next_name->name,
+							ldns_rr_rdf(
+								name->nsec, 0)) 
+						!= 0) {
 					printf("Error: the NSEC record for ");
 					ldns_rdf_print(stdout, name->name);
-					printf(" points to the wrong next owner name\n");
+					printf(" points to the wrong next "
+							"owner name\n");
 					if (verbosity >= 4) {
 						printf("     : ");
-						ldns_rdf_print(stdout,ldns_rr_rdf(name->nsec, 0));
+						ldns_rdf_print(stdout,
+							ldns_rr_rdf(name->nsec, 
+								0));
 						printf(" i.s.o. ");
-						ldns_rdf_print(stdout, next_name->name);
+						ldns_rdf_print(stdout, 
+							next_name->name);
 						printf(".\n");
 					}
 					if (result == LDNS_STATUS_OK) {
@@ -404,7 +422,8 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 				 * for this in the structs? (ie. pointer to next
 				 * hashed name?)
 				 */
-				status = verify_next_hashed_name(zone_nodes, name);
+				status = verify_next_hashed_name(
+						zone_nodes, name);
 				if (result == LDNS_STATUS_OK) {
 					result = status;
 				}
@@ -442,10 +461,10 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 
 static ldns_status
 verify_dnssec_name(ldns_rdf *zone_name,
-                ldns_dnssec_zone *zone,
+		ldns_dnssec_zone *zone,
                 ldns_rbtree_t *zone_nodes,
                 ldns_rbnode_t *cur_node,
-			    ldns_rr_list *keys)
+		ldns_rr_list *keys)
 {
 	ldns_status result = LDNS_STATUS_OK;
 	ldns_status status;
@@ -471,7 +490,8 @@ verify_dnssec_name(ldns_rdf *zone_name,
 					ldns_rdf_print(stdout, name->name);
 					printf("\t");
 					print_type(cur_rrset->type);
-					printf(" has signature(s), but is glue\n");
+					printf(" has signature(s), "
+							"but is glue\n");
 				}
 				result = LDNS_STATUS_ERR;
 			}
@@ -506,8 +526,10 @@ verify_dnssec_name(ldns_rdf *zone_name,
 					cur_rrset->type != LDNS_RR_TYPE_RRSIG
 				     && cur_rrset->type != LDNS_RR_TYPE_NSEC)) {
 
-				status = verify_dnssec_rrset(zone_name, name->name, cur_rrset, keys);
-				if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
+				status = verify_dnssec_rrset(zone_name,
+						name->name, cur_rrset, keys);
+				if (status != LDNS_STATUS_OK 
+						&& result == LDNS_STATUS_OK) {
 					result = status;
 				}
 			}
@@ -524,19 +546,18 @@ verify_dnssec_name(ldns_rdf *zone_name,
 
 static ldns_status
 verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
-			    ldns_rdf *zone_name) {
+		ldns_rdf *zone_name, bool apexonly, int percentage) {
+
 	ldns_rr_list *keys;
 	ldns_rbnode_t *cur_node;
 	ldns_dnssec_rrsets *cur_key_rrset;
 	ldns_dnssec_rrs *cur_key;
-	ldns_dnssec_name *cur_name;
 	ldns_status status;
 	ldns_status result = LDNS_STATUS_OK;
 
 	keys = ldns_rr_list_new();
-	cur_key_rrset = ldns_dnssec_zone_find_rrset(dnssec_zone,
-									    zone_name,
-									    LDNS_RR_TYPE_DNSKEY);
+	cur_key_rrset = ldns_dnssec_zone_find_rrset(
+				dnssec_zone, zone_name, LDNS_RR_TYPE_DNSKEY);
 	if (!cur_key_rrset || !cur_key_rrset->rrs) {
 		if (verbosity >= 1) {
 			printf("No DNSKEY records at zone apex\n");
@@ -560,20 +581,40 @@ verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
 			}
 			result = LDNS_STATUS_ERR;
 		}
+		if (apexonly) {
+			/*
+			 * In this case, only the first node in the treewalk
+			 * below should be checked.
+			 */
+			assert( cur_node->data == dnssec_zone->soa );
+			/* 
+			 * Allthough the percentage option doesn't make sense
+			 * here, we set it to 100 to force the first node to 
+			 * be checked.
+			 */
+			percentage = 100;
+		}
 		while (cur_node != LDNS_RBTREE_NULL) {
-			cur_name = (ldns_dnssec_name *) cur_node->data;
-			status = verify_dnssec_name(zone_name,
-			                            dnssec_zone,
-			                            dnssec_zone->names,
-			                            cur_node,
-			                            keys);
-			if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
-				result = status;
+			/* should we check this one? saves calls to random. */
+			if (percentage == 100
+					|| ((random() % 100) 
+						>= 100 - percentage)) {
+				status = verify_dnssec_name(
+						zone_name,
+						dnssec_zone,
+						dnssec_zone->names,
+						cur_node,
+						keys);
+				if (status != LDNS_STATUS_OK
+						&& result == LDNS_STATUS_OK) {
+					result = status;
+				}
+				if (apexonly)
+					break;
 			}
 			cur_node = ldns_rbtree_next(cur_node);
 		}
 	}
-
 	ldns_rr_list_free(keys);
 	return result;
 }
@@ -589,30 +630,50 @@ main(int argc, char **argv)
 	ldns_status s;
 	ldns_dnssec_zone *dnssec_zone;
 	ldns_status result = LDNS_STATUS_ERR;
+	bool apexonly = false;
+	int percentage = 100;
 
-	while ((c = getopt(argc, argv, "hvV:")) != -1) {
+	while ((c = getopt(argc, argv, "ahvV:p:")) != -1) {
 		switch(c) {
+                case 'a':
+                        apexonly = true;
+                        break;
 		case 'h':
 			printf("Usage: %s [OPTIONS] <zonefile>\n", argv[0]);
-			printf("\tReads the zonefile and checks for DNSSEC errors.\n");
+			printf("\tReads the zonefile and checks for DNSSEC "
+					"errors.\n");
 			printf("\nIt checks whether NSEC(3)s are present,");
 			printf(" and verifies all signatures\n");
-			printf("It also checks the NSEC(3) chain, but it will error on opted-out delegations\n");
+			printf("It also checks the NSEC(3) chain, but it will "
+					"error on opted-out delegations\n");
 			printf("\nOPTIONS:\n");
+			printf("\t-a apex only, check only the zone apex\n");
+			printf("\t-p [0-100] only checks this percentage of "
+					"the zone. Defaults to 100\n");
 			printf("\t-h show this text\n");
 			printf("\t-v shows the version and exits\n");
 			printf("\t-V [0-5]\tset verbosity level (default 3)\n");
-			printf("\nif no file is given standard input is read\n");
+			printf("\nif no file is given standard input is "
+					"read\n");
 			exit(EXIT_SUCCESS);
 			break;
 		case 'v':
-			printf("read zone version %s (ldns version %s)\n",
+			printf("verify-zone version %s (ldns version %s)\n",
 				  LDNS_VERSION, ldns_version());
 			exit(EXIT_SUCCESS);
 			break;
 		case 'V':
 			verbosity = atoi(optarg);
 			break;
+                case 'p':
+                        percentage = atoi(optarg);
+                        if (percentage < 0 || percentage > 100) {
+	                        fprintf(stderr, "percentage needs to fall "
+						"between 0..100\n");
+                                exit(EXIT_FAILURE);
+                        }
+                        srandom(time(NULL) ^ getpid());
+                        break;
 		}
 	}
 
@@ -635,7 +696,6 @@ main(int argc, char **argv)
 	}
 
 	s = ldns_zone_new_frm_fp_l(&z, fp, NULL, 0, LDNS_RR_CLASS_IN, &line_nr);
-
 	if (s == LDNS_STATUS_OK) {
 		if (!ldns_zone_soa(z)) {
 			fprintf(stderr, "; Error: no SOA in the zone\n");
@@ -646,7 +706,8 @@ main(int argc, char **argv)
 		result = ldns_dnssec_zone_mark_glue(dnssec_zone);
 		if (result != LDNS_STATUS_OK) {
 			if (verbosity >= 1) {
-				printf("There were errors identifying the glue in the zone\n");
+				printf("There were errors identifying the glue "
+						"in the zone\n");
 			}
 		}
 
@@ -654,9 +715,9 @@ main(int argc, char **argv)
 			ldns_dnssec_zone_print(stdout, dnssec_zone);
 		}
 
-		result = verify_dnssec_zone(dnssec_zone,
-							   ldns_rr_owner(ldns_zone_soa(z)));
-
+		result = verify_dnssec_zone(dnssec_zone, 
+				ldns_rr_owner(ldns_zone_soa(z)),
+				apexonly, percentage);
 
 		if (result == LDNS_STATUS_OK) {
 			if (verbosity >= 1) {
@@ -684,7 +745,8 @@ main(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
-	fprintf(stderr, "ldns-verifyzone needs OpenSSL support, which has not been compiled in\n");
+	fprintf(stderr, "ldns-verify-zone needs OpenSSL support, "
+			"which has not been compiled in\n");
 	return 1;
 }
 #endif /* HAVE_SSL */
