@@ -23,16 +23,20 @@
 ldns_lookup_table ldns_signing_algorithms[] = {
         { LDNS_SIGN_RSAMD5, "RSAMD5" },
         { LDNS_SIGN_RSASHA1, "RSASHA1" },
-        { LDNS_SIGN_RSASHA1_NSEC3, "RSASHA1_NSEC3" },
+        { LDNS_SIGN_RSASHA1_NSEC3, "RSASHA1-NSEC3-SHA1" },
 #ifdef USE_SHA2
         { LDNS_SIGN_RSASHA256, "RSASHA256" },
         { LDNS_SIGN_RSASHA512, "RSASHA512" },
 #endif
 #ifdef USE_GOST
         { LDNS_SIGN_ECC_GOST, "ECC-GOST" },
+#endif
+#ifdef USE_ECDSA
+        { LDNS_SIGN_ECDSAP256SHA256, "ECDSAP256SHA256" },
+        { LDNS_SIGN_ECDSAP384SHA384, "ECDSAP384SHA384" },
 #endif
         { LDNS_SIGN_DSA, "DSA" },
-        { LDNS_SIGN_DSA_NSEC3, "DSA_NSEC3" },
+        { LDNS_SIGN_DSA_NSEC3, "DSA-NSEC3-SHA1" },
         { LDNS_SIGN_HMACMD5, "hmac-md5.sig-alg.reg.int" },
         { LDNS_SIGN_HMACSHA1, "hmac-sha1" },
         { LDNS_SIGN_HMACSHA256, "hmac-sha256" },
@@ -91,14 +95,21 @@ ldns_key_new_frm_engine(ldns_key **key, ENGINE *e, char *key_id, ldns_algorithm
 	ldns_key *k;
 
 	k = ldns_key_new();
+        if(!k) return LDNS_STATUS_MEM_ERR;
+#ifndef S_SPLINT_S
 	k->_key.key = ENGINE_load_private_key(e, key_id, UI_OpenSSL(), NULL);
+        if(!k->_key.key) {
+                ldns_key_free(k);
+                return LDNS_STATUS_ERR;
+        }
 	ldns_key_set_algorithm(k, (ldns_signing_algorithm) alg);
 	if (!k->_key.key) {
+                ldns_key_free(k);
 		return LDNS_STATUS_ENGINE_KEY_NOT_LOADED;
-	} else {
-		*key = k;
-		return LDNS_STATUS_OK;
-	}
+	} 
+#endif /* splint */
+	*key = k;
+	return LDNS_STATUS_OK;
 }
 #endif
 
@@ -190,12 +201,92 @@ ldns_key_new_frm_fp_gost_l(FILE* fp, int* line_nr)
 	if(ldns_str2rdf_b64(&b64rdf, token) != LDNS_STATUS_OK)
 		return NULL;
 	pp = (unsigned char*)ldns_rdf_data(b64rdf);
-	pkey = d2i_PrivateKey(gost_id, NULL, &pp, ldns_rdf_size(b64rdf));
+	pkey = d2i_PrivateKey(gost_id, NULL, &pp, (int)ldns_rdf_size(b64rdf));
 	ldns_rdf_deep_free(b64rdf);
 	return pkey;
 }
 #endif
 
+#ifdef USE_ECDSA
+/** calculate public key from private key */
+static int
+ldns_EC_KEY_calc_public(EC_KEY* ec)
+{
+        EC_POINT* pub_key;
+        const EC_GROUP* group;
+        group = EC_KEY_get0_group(ec);
+        pub_key = EC_POINT_new(group);
+        if(!pub_key) return 0;
+        if(!EC_POINT_copy(pub_key, EC_GROUP_get0_generator(group))) {
+                EC_POINT_free(pub_key);
+                return 0;
+        }
+        if(!EC_POINT_mul(group, pub_key, EC_KEY_get0_private_key(ec),
+                NULL, NULL, NULL)) {
+                EC_POINT_free(pub_key);
+                return 0;
+        }
+        if(EC_KEY_set_public_key(ec, pub_key) == 0) {
+                EC_POINT_free(pub_key);
+                return 0;
+        }
+        EC_POINT_free(pub_key);
+        return 1;
+}
+
+/** read ECDSA private key */
+static EVP_PKEY*
+ldns_key_new_frm_fp_ecdsa_l(FILE* fp, ldns_algorithm alg, int* line_nr)
+{
+	char token[16384];
+        ldns_rdf* b64rdf = NULL;
+        unsigned char* pp;
+        BIGNUM* bn;
+        EVP_PKEY* evp_key;
+        EC_KEY* ec;
+	if (ldns_fget_keyword_data_l(fp, "PrivateKey", ": ", token, "\n", 
+		sizeof(token), line_nr) == -1)
+		return NULL;
+	if(ldns_str2rdf_b64(&b64rdf, token) != LDNS_STATUS_OK)
+		return NULL;
+        pp = (unsigned char*)ldns_rdf_data(b64rdf);
+
+        if(alg == LDNS_ECDSAP256SHA256)
+                ec = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
+        else if(alg == LDNS_ECDSAP384SHA384)
+                ec = EC_KEY_new_by_curve_name(NID_secp384r1);
+        else    ec = NULL;
+        if(!ec) {
+	        ldns_rdf_deep_free(b64rdf);
+                return NULL;
+        }
+	bn = BN_bin2bn(pp, (int)ldns_rdf_size(b64rdf), NULL);
+	ldns_rdf_deep_free(b64rdf);
+        if(!bn) {
+                EC_KEY_free(ec);
+                return NULL;
+        }
+        EC_KEY_set_private_key(ec, bn);
+        BN_free(bn);
+        if(!ldns_EC_KEY_calc_public(ec)) {
+                EC_KEY_free(ec);
+                return NULL;
+        }
+
+        evp_key = EVP_PKEY_new();
+        if(!evp_key) {
+                EC_KEY_free(ec);
+                return NULL;
+        }
+        if (!EVP_PKEY_assign_EC_KEY(evp_key, ec)) {
+		EVP_PKEY_free(evp_key);
+                EC_KEY_free(ec);
+                return NULL;
+	}
+        return evp_key;
+}
+#endif
+	
 ldns_status
 ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 {
@@ -214,6 +305,8 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 
 	d = LDNS_XMALLOC(char, LDNS_MAX_LINELEN);
 	if (!k || !d) {
+                ldns_key_free(k);
+                LDNS_FREE(d);
 		return LDNS_STATUS_MEM_ERR;
 	}
 
@@ -229,9 +322,13 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	if (ldns_fget_keyword_data_l(fp, "Private-key-format", ": ", d, "\n",
 				LDNS_MAX_LINELEN, line_nr) == -1) {
 		/* no version information */
+                ldns_key_free(k);
+                LDNS_FREE(d);
 		return LDNS_STATUS_SYNTAX_ERR;
 	}
 	if (strncmp(d, "v1.2", strlen(d)) != 0) {
+                ldns_key_free(k);
+                LDNS_FREE(d);
 		return LDNS_STATUS_SYNTAX_VERSION_ERR;
 	}
 
@@ -240,6 +337,8 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	if (ldns_fget_keyword_data_l(fp, "Algorithm", ": ", d, "\n",
 				LDNS_MAX_LINELEN, line_nr) == -1) {
 		/* no alg information */
+                ldns_key_free(k);
+                LDNS_FREE(d);
 		return LDNS_STATUS_SYNTAX_ALG_ERR;
 	}
 
@@ -289,6 +388,14 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		fprintf(stderr, "version of ldns, use --enable-gost\n");
 #endif
 	}
+#ifdef USE_ECDSA
+	if (strncmp(d, "13 ECDSAP256SHA256", 3) == 0) {
+                alg = LDNS_SIGN_ECDSAP256SHA256;
+        }
+	if (strncmp(d, "14 ECDSAP384SHA384", 3) == 0) {
+                alg = LDNS_SIGN_ECDSAP384SHA384;
+        }
+#endif
 	if (strncmp(d, "157 HMAC-MD5", 4) == 0) {
 		alg = LDNS_SIGN_HMACMD5;
 	}
@@ -304,7 +411,7 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	switch(alg) {
 		case LDNS_SIGN_RSAMD5:
 		case LDNS_SIGN_RSASHA1:
-		case LDNS_RSASHA1_NSEC3:
+		case LDNS_SIGN_RSASHA1_NSEC3:
 #ifdef USE_SHA2
 		case LDNS_SIGN_RSASHA256:
 		case LDNS_SIGN_RSASHA512:
@@ -321,7 +428,7 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 #endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_DSA:
-		case LDNS_DSA_NSEC3:
+		case LDNS_SIGN_DSA_NSEC3:
 			ldns_key_set_algorithm(k, alg);
 #ifdef HAVE_SSL
 			dsa = ldns_key_new_frm_fp_dsa_l(fp, line_nr);
@@ -356,16 +463,31 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
                         }
 			ldns_key_set_evp_key(k, 
 				ldns_key_new_frm_fp_gost_l(fp, line_nr));
+#ifndef S_SPLINT_S
 			if(!k->_key.key) {
 				ldns_key_free(k);
 				return LDNS_STATUS_ERR;
 			}
+#endif /* splint */
 #endif
 			break;
-		case 0:
-		default:
-			return LDNS_STATUS_SYNTAX_ALG_ERR;
+#ifdef USE_ECDSA
+               case LDNS_SIGN_ECDSAP256SHA256:
+               case LDNS_SIGN_ECDSAP384SHA384:
+                        ldns_key_set_algorithm(k, alg);
+                        ldns_key_set_evp_key(k,
+                                ldns_key_new_frm_fp_ecdsa_l(fp, (ldns_algorithm)alg, line_nr));
+#ifndef S_SPLINT_S
+			if(!k->_key.key) {
+				ldns_key_free(k);
+				return LDNS_STATUS_ERR;
+			}
+#endif /* splint */
 			break;
+#endif
+		default:
+			ldns_key_free(k);
+			return LDNS_STATUS_SYNTAX_ALG_ERR;
 	}
 	key_rr = ldns_key2rr(k);
 	ldns_key_set_keytag(k, ldns_calc_keytag(key_rr));
@@ -422,7 +544,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	buf = LDNS_XMALLOC(uint8_t, LDNS_MAX_LINELEN);
 	rsa = RSA_new();
 	if (!d || !rsa || !buf) {
-		return NULL;
+                goto error;
 	}
 
 	/* I could use functions again, but that seems an overkill,
@@ -434,6 +556,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 		goto error;
 	}
 	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
+#ifndef S_SPLINT_S
 	rsa->n = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!rsa->n) {
 		goto error;
@@ -508,6 +631,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	if (!rsa->iqmp) {
 		goto error;
 	}
+#endif /* splint */
 
 	LDNS_FREE(buf);
 	LDNS_FREE(d);
@@ -539,8 +663,8 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	d = LDNS_XMALLOC(char, LDNS_MAX_LINELEN);
 	buf = LDNS_XMALLOC(uint8_t, LDNS_MAX_LINELEN);
 	dsa = DSA_new();
-	if (!d || !dsa) {
-		return NULL;
+	if (!d || !dsa || !buf) {
+                goto error;
 	}
 
 	/* the line parser removes the () from the input... */
@@ -550,6 +674,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 		goto error;
 	}
 	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
+#ifndef S_SPLINT_S
 	dsa->p = BN_bin2bn((const char unsigned*)buf, i, NULL);
 	if (!dsa->p) {
 		goto error;
@@ -594,6 +719,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 	if (!dsa->pub_key) {
 		goto error;
 	}
+#endif /* splint */
 
 	LDNS_FREE(buf);
 	LDNS_FREE(d);
@@ -603,6 +729,7 @@ ldns_key_new_frm_fp_dsa_l(FILE *f, int *line_nr)
 error:
 	LDNS_FREE(d);
 	LDNS_FREE(buf);
+        DSA_free(dsa);
 	return NULL;
 }
 
@@ -623,6 +750,9 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 
 	d = LDNS_XMALLOC(char, LDNS_MAX_LINELEN);
 	buf = LDNS_XMALLOC(unsigned char, LDNS_MAX_LINELEN);
+        if(!d || !buf) {
+                goto error;
+        }
 
 	if (ldns_fget_keyword_data_l(f, "Key", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
@@ -683,6 +813,9 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 #ifdef HAVE_SSL
 	DSA *d;
 	RSA *r;
+#  ifdef USE_ECDSA
+        EC_KEY *ec = NULL;
+#  endif
 #else
 	int i;
 	uint16_t offset = 0;
@@ -701,10 +834,14 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 		case LDNS_SIGN_RSASHA512:
 #ifdef HAVE_SSL
 			r = RSA_generate_key((int)size, RSA_F4, NULL, NULL);
+                        if(!r) {
+				ldns_key_free(k);
+				return NULL;
+			}
 			if (RSA_check_key(r) != 1) {
+				ldns_key_free(k);
 				return NULL;
 			}
-
 			ldns_key_set_rsa_key(k, r);
 #endif /* HAVE_SSL */
 			break;
@@ -713,9 +850,11 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 #ifdef HAVE_SSL
 			d = DSA_generate_parameters((int)size, NULL, 0, NULL, NULL, NULL, NULL);
 			if (!d) {
+				ldns_key_free(k);
 				return NULL;
 			}
 			if (DSA_generate_key(d) != 1) {
+				ldns_key_free(k);
 				return NULL;
 			}
 			ldns_key_set_dsa_key(k, d);
@@ -725,12 +864,18 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
 #ifdef HAVE_SSL
+#ifndef S_SPLINT_S
 			k->_key.key = NULL;
+#endif /* splint */
 #endif /* HAVE_SSL */
 			size = size / 8;
 			ldns_key_set_hmac_size(k, size);
 
 			hmac = LDNS_XMALLOC(unsigned char, size);
+                        if(!hmac) {
+				ldns_key_free(k);
+				return NULL;
+                        }
 #ifdef HAVE_SSL
 			if (RAND_bytes(hmac, (int) size) != 1) {
 				LDNS_FREE(hmac);
@@ -755,8 +900,45 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 		case LDNS_SIGN_ECC_GOST:
 #if defined(HAVE_SSL) && defined(USE_GOST)
 			ldns_key_set_evp_key(k, ldns_gen_gost_key());
+#ifndef S_SPLINT_S
+                        if(!k->_key.key) {
+                                ldns_key_free(k);
+                                return NULL;
+                        }
+#endif /* splint */
 #endif /* HAVE_SSL and USE_GOST */
+                        break;
+#ifdef USE_ECDSA
+                case LDNS_SIGN_ECDSAP256SHA256:
+                case LDNS_SIGN_ECDSAP384SHA384:
+                        if(alg == LDNS_SIGN_ECDSAP256SHA256)
+                                ec = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
+                        else if(alg == LDNS_SIGN_ECDSAP384SHA384)
+                                ec = EC_KEY_new_by_curve_name(NID_secp384r1);
+                        if(!ec) {
+                                ldns_key_free(k);
+                                return NULL;
+                        }
+                        if(!EC_KEY_generate_key(ec)) {
+                                ldns_key_free(k);
+                                EC_KEY_free(ec);
+                                return NULL;
+                        }
+#ifndef S_SPLINT_S
+                        k->_key.key = EVP_PKEY_new();
+                        if(!k->_key.key) {
+                                ldns_key_free(k);
+                                EC_KEY_free(ec);
+                                return NULL;
+                        }
+                        if (!EVP_PKEY_assign_EC_KEY(k->_key.key, ec)) {
+                                ldns_key_free(k);
+                                EC_KEY_free(ec);
+                                return NULL;
+			}
+#endif /* splint */
 			break;
+#endif
 	}
 	ldns_key_set_algorithm(k, alg);
 	return k;
@@ -788,6 +970,7 @@ ldns_key_set_flags(ldns_key *k, uint16_t f)
 }
 
 #ifdef HAVE_SSL
+#ifndef S_SPLINT_S
 void
 ldns_key_set_evp_key(ldns_key *k, EVP_PKEY *e)
 {
@@ -809,6 +992,7 @@ ldns_key_set_dsa_key(ldns_key *k, DSA *d)
 	EVP_PKEY_set1_DSA(key, d);
 	k->_key.key  = key;
 }
+#endif /* splint */
 #endif /* HAVE_SSL */
 
 void
@@ -900,6 +1084,7 @@ ldns_key_use(const ldns_key *k)
 }
 
 #ifdef HAVE_SSL
+#ifndef S_SPLINT_S
 EVP_PKEY *
 ldns_key_evp_key(const ldns_key *k)
 {
@@ -925,6 +1110,7 @@ ldns_key_dsa_key(const ldns_key *k)
 		return NULL;
 	}
 }
+#endif /* splint */
 #endif /* HAVE_SSL */
 
 unsigned char *
@@ -1033,6 +1219,7 @@ ldns_key *
 ldns_key_list_pop_key(ldns_key_list *key_list)
 {                               
         size_t key_count;
+        ldns_key** a;
         ldns_key *pop;
 
 	if (!key_list) {
@@ -1047,8 +1234,10 @@ ldns_key_list_pop_key(ldns_key_list *key_list)
         pop = ldns_key_list_key(key_list, key_count);
         
         /* shrink the array */
-        key_list->_keys = LDNS_XREALLOC(
-                key_list->_keys, ldns_key *, key_count - 1);
+        a = LDNS_XREALLOC(key_list->_keys, ldns_key *, key_count - 1);
+        if(a) {
+                key_list->_keys = a;
+        }
 
         ldns_key_list_set_key_count(key_list, key_count - 1);
 
@@ -1056,6 +1245,8 @@ ldns_key_list_pop_key(ldns_key_list *key_list)
 }       
 
 #ifdef HAVE_SSL
+#ifndef S_SPLINT_S
+/* data pointer must be large enough (LDNS_MAX_KEYLEN) */
 static bool
 ldns_key_rsa2bin(unsigned char *data, RSA *k, uint16_t *size)
 {
@@ -1087,6 +1278,7 @@ ldns_key_rsa2bin(unsigned char *data, RSA *k, uint16_t *size)
 	return true;
 }
 
+/* data pointer must be large enough (LDNS_MAX_KEYLEN) */
 static bool
 ldns_key_dsa2bin(unsigned char *data, DSA *k, uint16_t *size)
 {
@@ -1136,6 +1328,7 @@ ldns_key_gost2bin(unsigned char* data, EVP_PKEY* k, uint16_t* size)
 	return true;
 }
 #endif /* USE_GOST */
+#endif /* splint */
 #endif /* HAVE_SSL */
 
 ldns_rr *
@@ -1154,6 +1347,9 @@ ldns_key2rr(const ldns_key *k)
 	RSA *rsa = NULL;
 	DSA *dsa = NULL;
 #endif /* HAVE_SSL */
+#ifdef USE_ECDSA
+        EC_KEY* ec;
+#endif
 	int internal_data = 0;
 
 	pubkey = ldns_rr_new();
@@ -1185,11 +1381,11 @@ ldns_key2rr(const ldns_key *k)
 
 	/* third - da algorithm */
 	switch(ldns_key_algorithm(k)) {
-		case LDNS_RSAMD5:
-		case LDNS_RSASHA1:
-		case LDNS_RSASHA1_NSEC3:
-		case LDNS_RSASHA256:
-		case LDNS_RSASHA512:
+		case LDNS_SIGN_RSAMD5:
+		case LDNS_SIGN_RSASHA1:
+		case LDNS_SIGN_RSASHA1_NSEC3:
+		case LDNS_SIGN_RSASHA256:
+		case LDNS_SIGN_RSASHA512:
 			ldns_rr_push_rdf(pubkey,
 						  ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
 #ifdef HAVE_SSL
@@ -1197,9 +1393,12 @@ ldns_key2rr(const ldns_key *k)
 			if (rsa) {
 				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
 				if (!bin) {
+                                        ldns_rr_free(pubkey);
 					return NULL;
 				}
 				if (!ldns_key_rsa2bin(bin, rsa, &size)) {
+		                        LDNS_FREE(bin);
+                                        ldns_rr_free(pubkey);
 					return NULL;
 				}
 				RSA_free(rsa);
@@ -1216,9 +1415,12 @@ ldns_key2rr(const ldns_key *k)
 			if (dsa) {
 				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
 				if (!bin) {
+                                        ldns_rr_free(pubkey);
 					return NULL;
 				}
 				if (!ldns_key_dsa2bin(bin, dsa, &size)) {
+		                        LDNS_FREE(bin);
+                                        ldns_rr_free(pubkey);
 					return NULL;
 				}
 				DSA_free(dsa);
@@ -1226,7 +1428,7 @@ ldns_key2rr(const ldns_key *k)
 			}
 #endif /* HAVE_SSL */
 			break;
-		case LDNS_DSA_NSEC3:
+		case LDNS_SIGN_DSA_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA_NSEC3));
 #ifdef HAVE_SSL
@@ -1234,9 +1436,12 @@ ldns_key2rr(const ldns_key *k)
 			if (dsa) {
 				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
 				if (!bin) {
+                                        ldns_rr_free(pubkey);
 					return NULL;
 				}
 				if (!ldns_key_dsa2bin(bin, dsa, &size)) {
+		                        LDNS_FREE(bin);
+                                        ldns_rr_free(pubkey);
 					return NULL;
 				}
 				DSA_free(dsa);
@@ -1249,19 +1454,57 @@ ldns_key2rr(const ldns_key *k)
 				LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
 #if defined(HAVE_SSL) && defined(USE_GOST)
 			bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
-			if (!bin) 
+			if (!bin) {
+                                ldns_rr_free(pubkey);
 				return NULL;
+                        }
+#ifndef S_SPLINT_S
 			if (!ldns_key_gost2bin(bin, k->_key.key, &size)) {
+		                LDNS_FREE(bin);
+                                ldns_rr_free(pubkey);
 				return NULL;
 			}
+#endif /* splint */
 			internal_data = 1;
-			break;
 #endif /* HAVE_SSL and USE_GOST */
+			break;
+#ifdef USE_ECDSA
+                case LDNS_SIGN_ECDSAP256SHA256:
+                case LDNS_SIGN_ECDSAP384SHA384:
+			ldns_rr_push_rdf(pubkey, ldns_native2rdf_int8(
+				LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
+                        bin = NULL;
+#ifndef S_SPLINT_S
+                        ec = EVP_PKEY_get1_EC_KEY(k->_key.key);
+#endif
+                        EC_KEY_set_conv_form(ec, POINT_CONVERSION_UNCOMPRESSED);
+                        size = (uint16_t)i2o_ECPublicKey(ec, NULL);
+                        if(!i2o_ECPublicKey(ec, &bin)) {
+                                EC_KEY_free(ec);
+                                ldns_rr_free(pubkey);
+                                return NULL;
+                        }
+			if(size > 1) {
+				/* move back one byte to shave off the 0x02
+				 * 'uncompressed' indicator that openssl made
+				 * Actually its 0x04 (from implementation).
+				 */
+				assert(bin[0] == POINT_CONVERSION_UNCOMPRESSED);
+				size -= 1;
+				memmove(bin, bin+1, size);
+			}
+                        /* down the reference count for ec, its still assigned
+                         * to the pkey */
+                        EC_KEY_free(ec);
+			internal_data = 1;
+                        break;
+#endif
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
 			bin = LDNS_XMALLOC(unsigned char, ldns_key_hmac_size(k));
 			if (!bin) {
+                                ldns_rr_free(pubkey);
 				return NULL;
 			}
 			ldns_rr_push_rdf(pubkey,
@@ -1325,6 +1568,9 @@ ldns_read_anchor_file(const char *filename)
 	size_t i = 0;
 	ldns_rr *r;
 	ldns_status status;
+        if(!line) {
+                return NULL;
+        }
 
 	fp = fopen(filename, "r");
 	if (!fp) {
@@ -1333,7 +1579,7 @@ ldns_read_anchor_file(const char *filename)
 		return NULL;
 	}
 	
-	while ((c = fgetc(fp)) && i < LDNS_MAX_PACKETLEN && c != EOF) {
+	while ((c = fgetc(fp)) && i+1 < LDNS_MAX_PACKETLEN && c != EOF) {
 		line[i] = c;
 		i++;
 	}
@@ -1386,3 +1632,41 @@ int ldns_key_algo_supported(int algo)
 	}
 	return 0;
 }
+
+ldns_signing_algorithm ldns_get_signing_algorithm_by_name(const char* name)
+{
+        /* list of (signing algorithm id, alias_name) */
+        ldns_lookup_table aliases[] = {
+                /* from bind dnssec-keygen */
+                {LDNS_SIGN_HMACMD5, "HMAC-MD5"},
+                {LDNS_SIGN_DSA_NSEC3, "NSEC3DSA"},
+                {LDNS_SIGN_RSASHA1_NSEC3, "NSEC3RSASHA1"},
+                /* old ldns usage, now RFC names */
+                {LDNS_SIGN_DSA_NSEC3, "DSA_NSEC3" },
+                {LDNS_SIGN_RSASHA1_NSEC3, "RSASHA1_NSEC3" },
+#ifdef USE_GOST
+                {LDNS_SIGN_ECC_GOST, "GOST"},
+#endif
+                /* compat with possible output */
+                {LDNS_DH, "DH"},
+                {LDNS_ECC, "ECC"},
+                {LDNS_INDIRECT, "INDIRECT"},
+                {LDNS_PRIVATEDNS, "PRIVATEDNS"},
+                {LDNS_PRIVATEOID, "PRIVATEOID"},
+                {0, NULL}};
+        ldns_lookup_table* lt = ldns_signing_algorithms;
+        while(lt->name) {
+                if(strcasecmp(lt->name, name) == 0)
+                        return lt->id;
+                lt++;
+        }
+        lt = aliases;
+        while(lt->name) {
+                if(strcasecmp(lt->name, name) == 0)
+                        return lt->id;
+                lt++;
+        }
+        if(atoi(name) != 0)
+                return atoi(name);
+        return 0;
+}
