@@ -256,8 +256,10 @@ ldns_resolver_pop_nameserver(ldns_resolver *r)
 	nameservers = LDNS_XREALLOC(nameservers, ldns_rdf *, (ns_count - 1));
 	rtt = LDNS_XREALLOC(rtt, size_t, (ns_count - 1));
 
-	ldns_resolver_set_nameservers(r, nameservers);
-	ldns_resolver_set_rtt(r, rtt);
+        if(nameservers)
+	        ldns_resolver_set_nameservers(r, nameservers);
+        if(rtt)
+	        ldns_resolver_set_rtt(r, rtt);
 	/* decr the count */
 	ldns_resolver_dec_nameserver_count(r);
 	return pop;
@@ -280,13 +282,26 @@ ldns_resolver_push_nameserver(ldns_resolver *r, ldns_rdf *n)
 	rtt = ldns_resolver_rtt(r);
 
 	/* make room for the next one */
-	nameservers = LDNS_XREALLOC(nameservers, ldns_rdf *, (ns_count + 1));
-	/* don't forget the rtt */
-	rtt = LDNS_XREALLOC(rtt, size_t, (ns_count + 1));
+	if (ns_count == 0) {
+		nameservers = LDNS_XMALLOC(ldns_rdf *, 1);
+	} else {
+		nameservers = LDNS_XREALLOC(nameservers, ldns_rdf *, (ns_count + 1));
+	}
+        if(!nameservers)
+                return LDNS_STATUS_MEM_ERR;
 
 	/* set the new value in the resolver */
 	ldns_resolver_set_nameservers(r, nameservers);
 
+	/* don't forget the rtt */
+	if (ns_count == 0) {
+		rtt = LDNS_XMALLOC(size_t, 1);
+	} else {
+		rtt = LDNS_XREALLOC(rtt, size_t, (ns_count + 1));
+	}
+        if(!rtt)
+                return LDNS_STATUS_MEM_ERR;
+
 	/* slide n in its slot. */
 	/* we clone it here, because then we can free the original
 	 * rr's where it stood */
@@ -326,6 +341,7 @@ ldns_resolver_push_nameserver_rr_list(ldns_resolver *r, ldns_rr_list *rrlist)
 			rr = ldns_rr_list_rr(rrlist, i);
 			if (ldns_resolver_push_nameserver_rr(r, rr) != LDNS_STATUS_OK) {
 				stat = LDNS_STATUS_ERR;
+				break;
 			}
 		}
 		return stat;
@@ -536,24 +552,27 @@ ldns_resolver_push_searchlist(ldns_resolver *r, ldns_rdf *d)
 
 		searchlist[list_count] = ldns_rdf_clone(d);
 		ldns_resolver_set_searchlist_count(r, list_count + 1);
-	}
+	} /* no way to report mem err */
 }
 
 void
 ldns_resolver_set_tsig_keyname(ldns_resolver *r, char *tsig_keyname)
 {
+	LDNS_FREE(r->_tsig_keyname);
 	r->_tsig_keyname = strdup(tsig_keyname);
 }
 
 void
 ldns_resolver_set_tsig_algorithm(ldns_resolver *r, char *tsig_algorithm)
 {
+	LDNS_FREE(r->_tsig_algorithm);
 	r->_tsig_algorithm = strdup(tsig_algorithm);
 }
 
 void
 ldns_resolver_set_tsig_keydata(ldns_resolver *r, char *tsig_keydata)
 {
+	LDNS_FREE(r->_tsig_keydata);
 	r->_tsig_keydata = strdup(tsig_keydata);
 }
 
@@ -639,7 +658,7 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 #ifdef HAVE_SSL
 	ldns_rr *tmp_rr;
 #endif
-	ssize_t gtr;
+	ssize_t gtr, bgtr;
 	ldns_buffer *b;
         int lnr = 0, oldline;
         if(!line_nr) line_nr = &lnr;
@@ -742,6 +761,11 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
                                         expect = LDNS_RESOLV_KEYWORD;
                                         continue;
                                 }
+                                if(strchr(word, '%')) {
+                                        /* snip off interface labels,
+                                         * fe80::222:19ff:fe31:4222%eth0 */
+                                        strchr(word, '%')[0]=0;
+                                }
 				tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_AAAA, word);
 				if (!tmp) {
 					/* try ip4 */
@@ -760,24 +784,37 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 				/* search list domain dname */
 				gtr = ldns_fget_token_l(fp, word, LDNS_PARSE_SKIP_SPACE, 0, line_nr);
 				b = LDNS_MALLOC(ldns_buffer);
+				if(!b) {
+					ldns_resolver_deep_free(r);
+					return LDNS_STATUS_MEM_ERR;
+				}
 
 				ldns_buffer_new_frm_data(b, word, (size_t) gtr);
-				gtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr + 1);
-				while (gtr > 0) {
+				if(ldns_buffer_status(b) != LDNS_STATUS_OK) {
+					LDNS_FREE(b);
+					ldns_resolver_deep_free(r);
+					return LDNS_STATUS_MEM_ERR;
+				}
+				bgtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr + 1);
+				while (bgtr > 0) {
+					gtr -= bgtr;
                                         if(word[0] == '#') {
                                                 expect = LDNS_RESOLV_KEYWORD;
+						ldns_buffer_free(b);
                                                 continue;
                                         }
 					tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, word);
 					if (!tmp) {
 						ldns_resolver_deep_free(r);
+						ldns_buffer_free(b);
 						return LDNS_STATUS_SYNTAX_DNAME_ERR;
 					}
 
 					ldns_resolver_push_searchlist(r, tmp);
 
 					ldns_rdf_deep_free(tmp);
-					gtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL, (size_t) gtr + 1);
+					bgtr = ldns_bget_token(b, word, LDNS_PARSE_NORMAL,
+					    (size_t) gtr + 1);
 				}
 				ldns_buffer_free(b);
 				gtr = 1;
@@ -797,6 +834,7 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 				/* a file containing a DNSSEC trust anchor */
 				gtr = ldns_fget_token_l(fp, word, LDNS_PARSE_NORMAL, 0, line_nr);
 				if (gtr == 0) {
+					ldns_resolver_deep_free(r);
 					return LDNS_STATUS_SYNTAX_MISSING_VALUE_ERR;
 				}
                                 if(word[0] == '#') {
@@ -818,6 +856,7 @@ ldns_resolver_new_frm_fp_l(ldns_resolver **res, FILE *fp, int *line_nr)
 		*res = r;
 		return LDNS_STATUS_OK;
 	} else {
+		ldns_resolver_deep_free(r);
 		return LDNS_STATUS_NULL;
 	}
 }
@@ -927,7 +966,12 @@ ldns_resolver_search(const ldns_resolver *r,const  ldns_rdf *name,
 			p = ldns_resolver_query(r, new_name, t, c, flags);
 			ldns_rdf_free(new_name);
 			if (p) {
-				return p;
+				if (ldns_pkt_get_rcode(p) == LDNS_RCODE_NOERROR) {
+					return p;
+				} else {
+					ldns_pkt_free(p);
+					p = NULL;
+				}
 			}
 		}
 	}
@@ -987,12 +1031,41 @@ ldns_resolver_query(const ldns_resolver *r, const ldns_rdf *name,
 	return pkt;
 }
 
+static size_t *
+ldns_resolver_backup_rtt(ldns_resolver *r)
+{
+	size_t *new_rtt;
+	size_t *old_rtt = ldns_resolver_rtt(r);
+
+	if (old_rtt && ldns_resolver_nameserver_count(r)) {
+		new_rtt = LDNS_XMALLOC(size_t
+				, ldns_resolver_nameserver_count(r));
+		memcpy(new_rtt, old_rtt, sizeof(size_t)
+				* ldns_resolver_nameserver_count(r));
+		ldns_resolver_set_rtt(r, new_rtt);
+		return old_rtt;
+	}
+	return NULL;
+}
+
+static void
+ldns_resolver_restore_rtt(ldns_resolver *r, size_t *old_rtt)
+{
+	size_t *cur_rtt = ldns_resolver_rtt(r);
+
+	if (cur_rtt) {
+		LDNS_FREE(cur_rtt);
+	}
+	ldns_resolver_set_rtt(r, old_rtt);
+}
+
 ldns_status
 ldns_resolver_send_pkt(ldns_pkt **answer, ldns_resolver *r,
 				   ldns_pkt *query_pkt)
 {
 	ldns_pkt *answer_pkt = NULL;
 	ldns_status stat = LDNS_STATUS_OK;
+	size_t *rtt;
 
 	stat = ldns_send(&answer_pkt, (ldns_resolver *)r, query_pkt);
 	if (stat != LDNS_STATUS_OK) {
@@ -1007,9 +1080,21 @@ ldns_resolver_send_pkt(ldns_pkt **answer, ldns_resolver *r,
 			if (ldns_pkt_tc(answer_pkt)) {
 				/* was EDNS0 set? */
 				if (ldns_pkt_edns_udp_size(query_pkt) == 0) {
-					ldns_pkt_set_edns_udp_size(query_pkt, 4096);
+					ldns_pkt_set_edns_udp_size(query_pkt
+							, 4096);
 					ldns_pkt_free(answer_pkt);
-					stat = ldns_send(&answer_pkt, r, query_pkt);
+					/* Nameservers should not become 
+					 * unreachable because fragments are
+					 * dropped (network error). We might
+					 * still have success with TCP.
+					 * Therefore maintain reachability
+					 * statuses of the nameservers by
+					 * backup and restore the rtt list.
+					 */
+					rtt = ldns_resolver_backup_rtt(r);
+					stat = ldns_send(&answer_pkt, r
+							, query_pkt);
+					ldns_resolver_restore_rtt(r, rtt);
 				}
 				/* either way, if it is still truncated, use TCP */
 				if (stat != LDNS_STATUS_OK ||
@@ -1035,6 +1120,8 @@ ldns_resolver_prepare_query_pkt(ldns_pkt **query_pkt, ldns_resolver *r,
                                 const ldns_rdf *name, ldns_rr_type t,
                                 ldns_rr_class c, uint16_t flags)
 {
+	struct timeval now;
+
 	/* prepare a question pkt from the parameters
 	 * and then send this */
 	*query_pkt = ldns_pkt_query_new(ldns_rdf_clone(name), t, c, flags);
@@ -1058,6 +1145,12 @@ ldns_resolver_prepare_query_pkt(ldns_pkt **query_pkt, ldns_resolver *r,
 		ldns_pkt_set_edns_udp_size(*query_pkt, ldns_resolver_edns_udp_size(r));
 	}
 
+	/* set the timestamp */
+	now.tv_sec = time(NULL);
+	now.tv_usec = 0;
+	ldns_pkt_set_timestamp(*query_pkt, now);
+
+
 	if (ldns_resolver_debug(r)) {
 		ldns_pkt_print(stdout, *query_pkt);
 	}
@@ -1112,8 +1205,8 @@ ldns_resolver_send(ldns_pkt **answer, ldns_resolver *r, const ldns_rdf *name,
 	  Jelte
 	  should this go in pkt_prepare?
 	*/
-#ifdef HAVE_SSL
 	if (ldns_resolver_tsig_keyname(r) && ldns_resolver_tsig_keydata(r)) {
+#ifdef HAVE_SSL
 		status = ldns_pkt_tsig_sign(query_pkt,
 		                            ldns_resolver_tsig_keyname(r),
 		                            ldns_resolver_tsig_keydata(r),
@@ -1121,10 +1214,10 @@ ldns_resolver_send(ldns_pkt **answer, ldns_resolver *r, const ldns_rdf *name,
 		if (status != LDNS_STATUS_OK) {
 			return LDNS_STATUS_CRYPTO_TSIG_ERR;
 		}
-	}
 #else
-	return LDNS_STATUS_CRYPTO_TSIG_ERR;
+	        return LDNS_STATUS_CRYPTO_TSIG_ERR;
 #endif /* HAVE_SSL */
+	}
 
 	status = ldns_resolver_send_pkt(&answer_pkt, r, query_pkt);
 	ldns_pkt_free(query_pkt);
@@ -1163,7 +1256,11 @@ ldns_axfr_next(ldns_resolver *resolver)
 		if (ldns_rr_get_type(cur_rr) == LDNS_RR_TYPE_SOA) {
 			resolver->_axfr_soa_count++;
 			if (resolver->_axfr_soa_count >= 2) {
+#ifndef USE_WINSOCK
 				close(resolver->_socket);
+#else
+				closesocket(resolver->_socket);
+#endif
 				resolver->_socket = 0;
 				ldns_pkt_free(resolver->_cur_axfr_pkt);
 				resolver->_cur_axfr_pkt = NULL;
@@ -1187,7 +1284,11 @@ ldns_axfr_next(ldns_resolver *resolver)
 			/* RoRi: we must now also close the socket, otherwise subsequent uses of the
 			   same resolver structure will fail because the link is still open or
 			   in an undefined state */
+#ifndef USE_WINSOCK
 			close(resolver->_socket);
+#else
+			closesocket(resolver->_socket);
+#endif
 			resolver->_socket = 0;
 
 			return NULL;
@@ -1198,7 +1299,11 @@ ldns_axfr_next(ldns_resolver *resolver)
 			/* RoRi: we must now also close the socket, otherwise subsequent uses of the
 			   same resolver structure will fail because the link is still open or
 			   in an undefined state */
+#ifndef USE_WINSOCK
 			close(resolver->_socket);
+#else
+			closesocket(resolver->_socket);
+#endif
 			resolver->_socket = 0;
 
 			return NULL;
