@@ -276,7 +276,7 @@ ldns_rr_dnskey_key_size_raw(const unsigned char* keydata,
 	uint16_t exp;
 	uint16_t int16;
 	
-	switch (alg) {
+	switch ((ldns_signing_algorithm)alg) {
 	case LDNS_SIGN_DSA:
 	case LDNS_SIGN_DSA_NSEC3:
 		if (len > 0) {
@@ -312,13 +312,17 @@ ldns_rr_dnskey_key_size_raw(const unsigned char* keydata,
 		}
 		break;
 #ifdef USE_GOST
-	case LDNS_SIGN_GOST:
+	case LDNS_SIGN_ECC_GOST:
 		return 512;
-		break;
+#endif
+#ifdef USE_ECDSA
+        case LDNS_SIGN_ECDSAP256SHA256:
+                return 256;
+        case LDNS_SIGN_ECDSAP384SHA384:
+                return 384;
 #endif
 	case LDNS_SIGN_HMACMD5:
 		return len;
-		break;
 	default:
 		return 0;
 	}
@@ -327,10 +331,11 @@ ldns_rr_dnskey_key_size_raw(const unsigned char* keydata,
 size_t 
 ldns_rr_dnskey_key_size(const ldns_rr *key) 
 {
-	if (!key) {
+	if (!key || !ldns_rr_dnskey_key(key) 
+			|| !ldns_rr_dnskey_algorithm(key)) {
 		return 0;
 	}
-	return ldns_rr_dnskey_key_size_raw(ldns_rdf_data(ldns_rr_dnskey_key(key)),
+	return ldns_rr_dnskey_key_size_raw((unsigned char*)ldns_rdf_data(ldns_rr_dnskey_key(key)),
 	                                   ldns_rdf_size(ldns_rr_dnskey_key(key)),
 	                                   ldns_rdf2native_int8(ldns_rr_dnskey_algorithm(key))
 	                                  );
